/* main.c - Application main entry point */

/*
 * Copyright (c) 2015-2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/types.h>
#include <stddef.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/util.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/controller.h>
#include <zephyr/bluetooth/hci.h>

static uint8_t mfg_data[] = { 0xff, 0xff, 0x00, 0x11, 0x22, 0x33, 0x44 };

static struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR),
	BT_DATA(BT_DATA_MANUFACTURER_DATA, mfg_data, sizeof(mfg_data)),
};

static K_SEM_DEFINE(adv_done, 0, 1);

static void bt_adv_done(struct bt_le_ext_adv *adv,
			struct bt_le_ext_adv_sent_info *info)
{
	ARG_UNUSED(adv);
	ARG_UNUSED(info);

	k_sem_give(&adv_done);
}

void main(void)
{
	int err;

	printk("Starting Advertiser Demo\n");

	/* Initialize the Bluetooth Subsystem */
	err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}

	printk("Bluetooth initialized\n");

	struct bt_le_adv_param *adv_param = BT_LE_ADV_PARAM(BT_LE_ADV_OPT_USE_IDENTITY | BT_LE_ADV_OPT_EXT_ADV, 
														BT_GAP_ADV_FAST_INT_MIN_1, BT_GAP_ADV_FAST_INT_MAX_1, NULL);
	struct bt_le_ext_adv *adv;
	struct bt_le_ext_adv_cb adv_cb = {
		.sent = bt_adv_done
	};

	/* Create advertising set */
	err = bt_le_ext_adv_create(adv_param, &adv_cb, &adv);
	if (err) {
		printk("Failed to create advertising set (err %d)\n", err);
		return;
	}

	do {
		printk("TX: %d\n", mfg_data[2]);

		/* Swap between different lengths */
		if (ad[1].data_len % 2) {
			ad[1].data_len = 4;
		} else {
			ad[1].data_len = 7;
		}

		/* Set the advertising data */
		err = bt_le_ext_adv_set_data(adv, ad, ARRAY_SIZE(ad), NULL, 0);
		if (err) {
			printk("Failed to set advertising data (err %d)\n", err);
			return;
		}

		/* Start advertising */
		err = bt_le_ext_adv_start(adv, BT_LE_EXT_ADV_START_PARAM(0, 1));
		if (err) {
			printk("Advertising failed to start (err %d)\n", err);
			return;
		}

		/* Wait for advertising to complete */
		if (k_sem_take(&adv_done, K_SECONDS(1)) != 0) {
			printk("Advertising failed to complete)\n");
			return;
		}

		mfg_data[2]++;

		k_sleep(K_MSEC(1000));
	} while (1);
}
